==> iwrite-1.0.1.4.tar.gz: iwrite-1.0.1.4/src/item_image.c <==
/* 
vim:expandtab:softtabstop=2:tabstop=2:shiftwidth=2:nowrap:ruler
*/
/*
Copyright (c) 2015-2016, iwrite authors 
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include "item_image.h"

extern void
item_image_assign(
  struct item_image*const               o_image,
  struct item_method*const              o_method)
{

  memset(o_image, 0, sizeof(*o_image));

  (*o_image).m_len_x= 30.0;
  (*o_image).m_len_y= 30.0;
  (*o_image).m_border_thickness= 1.0;

  ITEM_METHOD_ASSIGN(o_method, image);

  return;
}

extern void
item_image_destroy(
  struct item_image*const                io_image)
{

  if ((*io_image).m_image) 
  {
    g_object_unref((*io_image).m_image);
    (*io_image).m_image= 0;
  }

  if ((*io_image).m_image_sized) 
  {
    g_object_unref((*io_image).m_image_sized);
    (*io_image).m_image_sized= 0;
  }

  return;
}

extern void
item_image_discharge(
  struct item_image*const                io_image)
{

  item_image_destroy(io_image);
  memset(io_image, 0, sizeof(*io_image));

  return;
}

extern void
item_image_draw(
  struct item_image*const               i_image,
  struct item_draw_context const*const  i_ctx)
{

  do
  {

    if ((*i_image).m_border)
    {
      item_common_cairo_dash_set((*i_ctx).m_cr, (*i_image).m_border_style);
      cairo_set_line_width((*i_ctx).m_cr, (*i_image).m_border_thickness);
      cairo_set_source_rgb(
        (*i_ctx).m_cr,
        (*i_image).m_border_color.red,
        (*i_image).m_border_color.green,
        (*i_image).m_border_color.blue);
      cairo_rectangle(
        (*i_ctx).m_cr,
        (*i_image).m_pos_x,
        (*i_image).m_pos_y,
        (*i_image).m_len_x,
        (*i_image).m_len_y);
      cairo_stroke((*i_ctx).m_cr);
    }

    if ((*i_image).m_image_sized)
    {
      gdk_cairo_set_source_pixbuf(
        (*i_ctx).m_cr, 
        (*i_image).m_image_sized,
        (*i_image).m_pos_x,
        (*i_image).m_pos_y);

      cairo_paint((*i_ctx).m_cr);
    }
    else if ((item_draw_design == (*i_ctx).m_option.m_type) && 
      (0 == (*i_image).m_border))
    {
      item_common_cairo_dash_set((*i_ctx).m_cr, line_style_dash);
      cairo_set_line_width((*i_ctx).m_cr, 1.0);
      cairo_set_source_rgb((*i_ctx).m_cr, 0.0, 0.0, 0.0);
      cairo_rectangle(
        (*i_ctx).m_cr,
        (*i_image).m_pos_x,
        (*i_image).m_pos_y,
        (*i_image).m_len_x,
        (*i_image).m_len_y);
      cairo_stroke((*i_ctx).m_cr);
    }

    if (0 == (*i_ctx).m_selected)
    {
      if ((*i_ctx).m_option.m_highlight &&
        (item_draw_design == (*i_ctx).m_option.m_type))
      {
        item_common_box_draw_hightlight(
          (*i_ctx).m_cr, 
          (struct geom_rect*)i_image,
          (*i_image).m_border_thickness);
      }
      break;
    }

    item_common_box_draw_grips((*i_ctx).m_cr, (struct geom_rect*)i_image);

  }while(0);

  return;
}

extern void
item_image_get_bbox(
  struct geom_rect*const                o_bbox,
  int*const                             o_overlay,
  void const*const                      i_object)
{
  struct item_image const*              l_image;
  l_image= (struct item_image const*)i_object;
  (*o_bbox)= *(struct geom_rect*)i_object;
  (*o_overlay)= (*l_image).m_overlay;
  return;
}

extern void
item_image_in_event(
  enum event_type*const                 o_type,
  struct item_image const*const         i_image,
  struct item_in_event const*const      i_ctx)
{

  item_common_box_in_event(o_type, i_ctx, (struct geom_rect*)i_image);

  return;
}

extern int
item_image_load(
  GError**                              o_error,
  struct item_image*const               io_image,
  char const*                           i_filename)
{
  GError*                               l_error;
  int                                   l_exit;

  l_error= 0;
  l_exit= 0;

  do
  {

    if ((0 == i_filename) || (0 == i_filename[0]))
    {
      break;
    }

    item_image_destroy(io_image);

    (*io_image).m_image= gdk_pixbuf_new_from_file(i_filename, &l_error);

    if (l_error)
    {
      _error_log(l_error);
      l_exit= -1;
      break;
    }

    (*io_image).m_image_sized= gdk_pixbuf_scale_simple(
      (*io_image).m_image,
      (*io_image).m_len_x,
      (*io_image).m_len_y,
      GDK_INTERP_BILINEAR);

  }while(0);

  if (l_error)
  {
    g_propagate_error(o_error, l_error);
  }

  return l_exit;
}

extern void
item_image_move(
  struct item_image*const               io_image,
  struct geom_point const*const         i_event)
{
  item_common_box_move((struct geom_rect*)io_image, i_event);
  return;
}

extern void
item_image_normalize(
  struct item_image*const               io_image,
  double const                          i_bound_width,
  double const                          i_bound_height)
{

  item_common_box_normalize(
    (struct geom_rect*)io_image, 
    i_bound_width,
    i_bound_height);

  do
  {

    if (0 == (*io_image).m_image)
    {
      break;
    }

    if ((*io_image).m_image_sized)
    {
      g_object_unref((*io_image).m_image_sized);
      (*io_image).m_image_sized= 0;
    }

    (*io_image).m_image_sized= gdk_pixbuf_scale_simple(
      (*io_image).m_image,
      (*io_image).m_len_x,
      (*io_image).m_len_y,
      GDK_INTERP_BILINEAR);

  }while(0);

  return;
}

extern int
item_image_read(
  GError**                              o_error,
  struct item_image*const               io_image,
  struct bson_node *const               i_element)
{
  struct bson_node *                    l_element;
  int                                   l_found;
  enum element_tag_type                 l_type;

  l_element= i_element;

  do
  {

    if (0 == l_element)
    {
      break;
    }

    if (bson_type_end == (*l_element).m_type)
    {
      break;
    }

    item_common_lookup(&l_found, &l_type, (*l_element).m_ename);

    if (l_found)
    {
        switch(l_type)
        {
          case element_tag_border:
            if (bson_type_int32 == (*l_element).m_type)
            {
              (*io_image).m_border= (*l_element).m_object.m_int32;
            }
            break;
          case element_tag_border_color_blue:
            if (bson_type_float == (*l_element).m_type)
            {
              (*io_image).m_border_color.blue= (*l_element).m_object.m_float;
            }
            break;
          case element_tag_border_color_green:
            if (bson_type_float == (*l_element).m_type)
            {
              (*io_image).m_border_color.green= (*l_element).m_object.m_float;
            }
            break;
          case element_tag_border_color_red:
            if (bson_type_float == (*l_element).m_type)
            {
              (*io_image).m_border_color.red= (*l_element).m_object.m_float;
            }
            break;
          case element_tag_border_style:
            if (bson_type_int32 == (*l_element).m_type)
            {
              (*io_image).m_border_style= (*l_element).m_object.m_int32;
            }
            break;
          case element_tag_border_thickness:
            if (bson_type_float == (*l_element).m_type)
            {
              (*io_image).m_border_thickness= (*l_element).m_object.m_float;
            }
            break;
          case element_tag_len_x:
            if (bson_type_float == (*l_element).m_type)
            {
              (*io_image).m_len_x= (*l_element).m_object.m_float;
            }
            break;
          case element_tag_len_y:
            if (bson_type_float == (*l_element).m_type)
            {
              (*io_image).m_len_y= (*l_element).m_object.m_float;
            }
            break;
          case element_tag_pos_x:
            if (bson_type_float == (*l_element).m_type)
            {
              (*io_image).m_pos_x= (*l_element).m_object.m_float;
            }
            break;
          case element_tag_pos_y:
            if (bson_type_float == (*l_element).m_type)
            {
              (*io_image).m_pos_y= (*l_element).m_object.m_float;
            }
            break;
          case element_tag_overlay:
            if (bson_type_int32 == (*l_element).m_type)
            {
              (*io_image).m_overlay= (*l_element).m_object.m_int32;
            }
            break; 
          case element_tag_pixbuf:
            if (bson_type_binary == (*l_element).m_type)
            {
              if ((*io_image).m_image) 
              {
                g_object_unref((*io_image).m_image);
                (*io_image).m_image= 0;
              }
              pixbuf_read(&(*io_image).m_image, l_element);
              if ((*io_image).m_image)
              {
                (*io_image).m_image_sized= gdk_pixbuf_scale_simple(
                  (*io_image).m_image,
                  (*io_image).m_len_x,
                  (*io_image).m_len_y,
                  GDK_INTERP_BILINEAR);
              }
            }
            break;
          default:
            break;
        }
    }

    l_element= (*l_element).m_next;

  }while(1);

  return 0;
}

extern void
item_image_resize(
  struct item_image*const               io_image,
  struct item_resize_event const*const  i_ctx)
{

  item_common_box_resize((struct geom_rect *)io_image, i_ctx);

  return;
}

extern int
item_image_write(
  GError**                              o_error,
  FILE*const                            io_fp,
  struct item_image const*const         i_image)
{
  int                                   l_exit;
  int32_t                               l_int32;
  char                                  l_octet;
  int32_t                               l_offset;
  int32_t                               l_size;

  l_exit= 0;

  /* array */
  l_octet= bson_type_array;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "image%c",0);

  /* document */
  l_offset= ftell(io_fp);
  l_size= 0;
  fwrite(&l_size, sizeof(int32_t), 1, io_fp); 

  /* pos_x */
  l_octet= bson_type_float;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "pos_x%c",0);
  fwrite(&(*i_image).m_pos_x, sizeof(double), 1, io_fp);

  /* pos_y */
  l_octet= bson_type_float;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "pos_y%c",0);
  fwrite(&(*i_image).m_pos_y, sizeof(double), 1, io_fp);

  /* len_x */
  l_octet= bson_type_float;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "len_x%c",0);
  fwrite(&(*i_image).m_len_x, sizeof(double), 1, io_fp);

  /* len_y */
  l_octet= bson_type_float;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "len_y%c",0);
  fwrite(&(*i_image).m_len_y, sizeof(double), 1, io_fp);

  /* border */
  l_octet= bson_type_int32;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "border%c",0);
  fwrite(&(*i_image).m_border, sizeof(int32_t), 1, io_fp);

  /* border_style */
  l_octet= bson_type_int32;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "border_style%c",0);
  fwrite(&(*i_image).m_border_style, sizeof(int32_t), 1, io_fp);

  /* border_thickness */
  l_octet= bson_type_float;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "border_thickness%c",0);
  fwrite(&(*i_image).m_border_thickness, sizeof(double), 1, io_fp);

  /* border_color - red */
  l_octet= bson_type_float;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "border_color_red%c",0);
  fwrite(&(*i_image).m_border_color.red, sizeof(double), 1, io_fp);

  /* border_color - green */
  l_octet= bson_type_float;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "border_color_green%c",0);
  fwrite(&(*i_image).m_border_color.green, sizeof(double), 1, io_fp);

  /* border_color - blue */
  l_octet= bson_type_float;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "border_color_blue%c",0);
  fwrite(&(*i_image).m_border_color.blue, sizeof(double), 1, io_fp);

  /* overlay */
  l_octet= bson_type_int32;
  fwrite(&l_octet, 1, 1, io_fp); 
  fprintf(io_fp, "overlay%c",0);
  fwrite(&(*i_image).m_overlay, sizeof(int32_t), 1, io_fp);

  /* image */
  if ((*i_image).m_image)
  {
    pixbuf_write(io_fp, (*i_image).m_image);
  }

  /* end of document */
  l_octet= 0;
  fwrite(&l_octet, 1, 1, io_fp); 

  /* document */
  l_int32= ftell(io_fp);
  l_size= (l_int32 - l_offset);
  fseek(io_fp, l_offset, SEEK_SET);
  fwrite(&l_size, sizeof(l_size), 1, io_fp); 


  fseek(io_fp, 0, SEEK_END);

  return l_exit;
}

==> iwrite-2.0.1.0.tar.gz: iwrite-2.0.1.0/src/item_image.c <==
/* 
vim:expandtab:softtabstop=2:tabstop=2:shiftwidth=2:nowrap:ruler
*/
/*
Copyright (c) 2015-2016, iwrite authors 
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include "item_image.h"

extern void
item_image_assign(
  struct item_image*const               o_image,
  struct item_method*const              o_method)
{

  memset(o_image, 0, sizeof(*o_image));

  (*o_image).m_len_x= 30.0;
  (*o_image).m_len_y= 30.0;
  (*o_image).m_border.m_thickness= 1.0;
  (*o_image).m_border.m_color.alpha= 1.0;

  ITEM_METHOD_ASSIGN(o_method, image);

  return;
}

extern void
item_image_destroy(
  struct item_image*const                io_image)
{

  if ((*io_image).m_image) 
  {
    g_object_unref((*io_image).m_image);
    (*io_image).m_image= 0;
  }

  if ((*io_image).m_image_sized) 
  {
    g_object_unref((*io_image).m_image_sized);
    (*io_image).m_image_sized= 0;
  }

  return;
}

extern void
item_image_discharge(
  struct item_image*const                io_image)
{

  item_image_destroy(io_image);
  memset(io_image, 0, sizeof(*io_image));

  return;
}

extern void
item_image_draw(
  struct item_image*const               i_image,
  struct item_draw_context const*const  i_ctx)
{

  do
  {

    if ((*i_image).m_border.m_active)
    {
      item_common_cairo_dash_set((*i_ctx).m_cr, (*i_image).m_border.m_style);
      cairo_set_line_width((*i_ctx).m_cr, (*i_image).m_border.m_thickness);
      cairo_set_source_rgba(
        (*i_ctx).m_cr,
        (*i_image).m_border.m_color.red,
        (*i_image).m_border.m_color.green,
        (*i_image).m_border.m_color.blue,
        (*i_image).m_border.m_color.alpha);
      cairo_rectangle(
        (*i_ctx).m_cr,
        (*i_image).m_pos_x,
        (*i_image).m_pos_y,
        (*i_image).m_len_x,
        (*i_image).m_len_y);
      cairo_stroke((*i_ctx).m_cr);
    }

    if ((*i_image).m_image_sized)
    {
      gdk_cairo_set_source_pixbuf(
        (*i_ctx).m_cr, 
        (*i_image).m_image_sized,
        (*i_image).m_pos_x,
        (*i_image).m_pos_y);

      cairo_paint((*i_ctx).m_cr);
    }
    else if ((item_draw_design == (*i_ctx).m_option.m_type) && 
      (0 == (*i_image).m_border.m_active))
    {
      item_common_cairo_dash_set((*i_ctx).m_cr, line_style_dash);
      cairo_set_line_width((*i_ctx).m_cr, 1.0);
      cairo_set_source_rgb((*i_ctx).m_cr, 0.0, 0.0, 0.0);
      cairo_rectangle(
        (*i_ctx).m_cr,
        (*i_image).m_pos_x,
        (*i_image).m_pos_y,
        (*i_image).m_len_x,
        (*i_image).m_len_y);
      cairo_stroke((*i_ctx).m_cr);
    }

    if (0 == (*i_ctx).m_selected)
    {
      if ((*i_ctx).m_option.m_highlight &&
        (item_draw_design == (*i_ctx).m_option.m_type))
      {
        item_common_box_draw_hightlight(
          (*i_ctx).m_cr, 
          (struct geom_rect*)i_image,
          1.0);
      }
      break;
    }

    item_common_box_draw_grips((*i_ctx).m_cr, (struct geom_rect*)i_image);

  }while(0);

  return;
}

extern void
item_image_get_bbox(
  struct geom_rect*const                o_bbox,
  void const*const                      i_object)
{
  (*o_bbox)= *(struct geom_rect*)i_object;
  return;
}

extern void
item_image_in_event(
  enum event_type*const                 o_type,
  struct item_image const*const         i_image,
  struct item_in_event const*const      i_ctx)
{

  item_common_box_in_event(o_type, i_ctx, (struct geom_rect*)i_image);

  return;
}

extern int
item_image_load(
  GError**                              o_error,
  struct item_image*const               io_image,
  char const*                           i_filename)
{
  GError*                               l_error;
  int                                   l_exit;

  l_error= 0;
  l_exit= 0;

  do
  {

    if ((0 == i_filename) || (0 == i_filename[0]))
    {
      break;
    }

    item_image_destroy(io_image);

    (*io_image).m_image= gdk_pixbuf_new_from_file(i_filename, &l_error);

    if (l_error)
    {
      _error_log(l_error);
      l_exit= -1;
      break;
    }

    (*io_image).m_image_sized= gdk_pixbuf_scale_simple(
      (*io_image).m_image,
      (*io_image).m_len_x,
      (*io_image).m_len_y,
      GDK_INTERP_BILINEAR);

  }while(0);

  if (l_error)
  {
    g_propagate_error(o_error, l_error);
  }

  return l_exit;
}

extern void
item_image_move(
  struct item_image*const               io_image,
  struct geom_point const*const         i_event)
{
  item_common_box_move((struct geom_rect*)io_image, i_event);
  return;
}

extern void
item_image_normalize(
  struct item_image*const               io_image,
  double const                          i_bound_width,
  double const                          i_bound_height)
{

  item_common_box_normalize(
    (struct geom_rect*)io_image, 
    i_bound_width,
    i_bound_height);

  do
  {

    if (0 == (*io_image).m_image)
    {
      break;
    }

    if ((*io_image).m_image_sized)
    {
      g_object_unref((*io_image).m_image_sized);
      (*io_image).m_image_sized= 0;
    }

    (*io_image).m_image_sized= gdk_pixbuf_scale_simple(
      (*io_image).m_image,
      (*io_image).m_len_x,
      (*io_image).m_len_y,
      GDK_INTERP_BILINEAR);

  }while(0);

  return;
}

extern int
item_image_read(
  GError**                              o_error,
  struct item_image*const               io_image,
  struct xml *const                     i_xml)
{
  GError*                               l_error;
  int                                   l_exit;
  int                                   l_found;
  GList*                                l_node;
  enum element_tag_type                 l_type;
  char const*                           l_value;
  struct xml*                           l_xml;

  l_error= 0;
  l_exit= 0;

  l_node= (*i_xml).m_children;

  do
  {

    if (0 == l_node)
    {
      break;
    }

    l_xml= (*l_node).data;

    item_common_lookup(&l_found, &l_type, (*l_xml).m_tag);
    g_strstrip((*l_xml).m_string->str);
    l_value= (*l_xml).m_string->str;

    if (l_found)
    {
      switch(l_type)
      {
        case element_tag_pos_x:
          (*io_image).m_pos_x= xml_convert_double(l_value);
          break;
        case element_tag_pos_y:
          (*io_image).m_pos_y= xml_convert_double(l_value);
          break;
        case element_tag_len_x:
          (*io_image).m_len_x= xml_convert_double(l_value);
          break;
        case element_tag_len_y:
          (*io_image).m_len_y= xml_convert_double(l_value);
          break;
        case element_tag_border:
          l_exit= item_common_read_border_attr(&l_error, &(*io_image).m_border, l_xml);
          break;
        case element_tag_data:
          l_exit= pixbuf_read(&l_error, &(*io_image).m_image, l_xml);
        default:
          break;
      }
    }

    l_node= (*l_node).next;

  }while(1);

  if (0 == l_exit && (*io_image).m_image)
  {
    (*io_image).m_image_sized= gdk_pixbuf_scale_simple(
      (*io_image).m_image,
      (*io_image).m_len_x,
      (*io_image).m_len_y,
      GDK_INTERP_BILINEAR);
  }

  if (l_error)
  {
    g_propagate_error(o_error, l_error);
  }

  return l_exit;
}

extern void
item_image_resize(
  struct item_image*const               io_image,
  struct item_resize_event const*const  i_ctx)
{

  item_common_box_resize((struct geom_rect *)io_image, i_ctx);

  return;
}

extern int
item_image_write(
  GError**                              o_error,
  FILE*const                            io_fp,
  struct item_image const*const         i_image)
{
  char                                  l_buf[G_ASCII_DTOSTR_BUF_SIZE+1];
  int                                   l_exit;

  l_exit= 0;

  fprintf(io_fp, "\t\t\t<item name=\"image\">\n");
  fprintf(io_fp, "\t\t\t\t<pos_x>%s</pos_x>\n", g_ascii_dtostr(l_buf, sizeof(l_buf), (*i_image).m_pos_x));
  fprintf(io_fp, "\t\t\t\t<pos_y>%s</pos_y>\n", g_ascii_dtostr(l_buf, sizeof(l_buf), (*i_image).m_pos_y));
  fprintf(io_fp, "\t\t\t\t<len_x>%s</len_x>\n", g_ascii_dtostr(l_buf, sizeof(l_buf), (*i_image).m_len_x));
  fprintf(io_fp, "\t\t\t\t<len_y>%s</len_y>\n", g_ascii_dtostr(l_buf, sizeof(l_buf), (*i_image).m_len_y));
  item_common_write_border_attr(o_error, io_fp, &(*i_image).m_border);

  if ((*i_image).m_image)
  {
    pixbuf_write(io_fp, (*i_image).m_image);
  }

  fprintf(io_fp, "\t\t\t</item>\n");

  return l_exit;
}
